#ifndef __FDC_H_INCLUDED__
#define __FDC_H_INCLUDED__

typedef struct fdc_geom{
	int8_t		heads;
	int8_t		tracks;
	int8_t		spt;		/* Sectors per track */
	int8_t		g3_fmt;		/* GAP3 while formatting */
	int8_t		g3_rw;		/* GAP3 while read/writing */
}fdc_geom_t, *p_fdc_geom;

/* Driver stuff */
void fdc_init(void);
u_int32_t fdc_probe(void);
u_int32_t fdc_setup(void);

/* Low level block driver */
int32_t fdc_blk_rw(int32_t, u_int32_t, char *, u_int32_t);

/* Floppy functions */
void fdc_reset(void);	
int  fdc_seek(int8_t);
void fdc_recalibrate(void);

int fdc_read(u_int32_t, char *, int len);

/* Utility functions */
void fdc_linear2hts(u_int32_t block, int *, int *, int *);
int8_t fdc_getbyte(void);
void fdc_sendbyte(int8_t);

/* FDC I/O ports */
#define	FDC_DSRA	0x3f0		/* Diskette status register A (PS/2) */
#define FDC_DSRB	0x3f1		/* Diskette status register B (PS/2) */
#define FDC_DOR		0x3f2		/* Digital output register (All) */
#define FDC_MSR		0x3f4		/* Main status register (All) RO */
#define FDC_DRS		0x3f4		/* Data rate select (All) WO */
#define FDC_DATA	0x3f5		/* Data register */
#define FDC_DIR		0x3f7		/* Digital Input register (PS/2) RO */
#define FDC_CCR		0x3f7		/* Configuration Control register (PS/2) WO */

/* FDC command codes */
#define CMD_SPECIFY	0x03
#define CMD_WRITE	0xc5
#define CMD_READ	0xe6
#define CMD_RECALIBRATE	0x07
#define CMD_SENSEI	0x08
#define CMD_FORMAT	0x4d
#define CMD_SEEK	0x0f
#define CMD_VERSION	0x10

#define FDC_GOT_TIME		(ticks<fdc_timeout)
#define FDC_SET_TIMEOUT(x)	(fdc_timeout=ticks+x)

#endif /* __FDC_H_INCLUDED__ */
