#ifndef __ARCH_IA32_MM__
#define __ARCH_IA32_MM__

#include <config.h>
#include <types.h>

/* Characteristics of the IA32 architecture */
#define MAX_PAGE_DIR_ENTRIES	1024
#define MAX_PAGE_TBL_ENTRIES	1024
#define PAGE_SIZE		4096

/* Macros to mask top 10, middle 10, and bottom 12 bits of
* an address respectively */
#define __dir(x)		(x&0xFFC00000)
#define __tbl(x)		(x&0x003FF000)
#define __ofs(x)		(x&0x00000FFF)

/* Get the actual indexes */
#define dir(x)			(__dir(x)>>22)
#define tbl(x)			(__tbl(x)>>12)
#define ofs(x)			(__ofs(x))

/* Generate a virtual address */
#define __mkvirt(d,t,o)		(__dir(d<<22) | __tbl(t<<12) | __ofs(o))

/* Macros to seperate value and flags from pdes and ptes */
#define __val(x)		(x&0xFFFFF000)
#define __flags(x)		(x&0x00000FFF)

typedef struct pde {
	u_int32_t value;
}pde_t, *p_pde;

typedef struct pte {
	u_int32_t value;
}pte_t, *p_pte;

typedef struct pagedir {
	pde_t		entry[MAX_PAGE_DIR_ENTRIES];
}pagedir_t, *p_pagedir;

typedef struct pagetbl {
	pte_t		entry[MAX_PAGE_TBL_ENTRIES];
}pagetbl_t, *p_pagetbl;

enum {
	PDE_PRESENT,
	PDE_RW,
	PDE_SUPERVISOR,
	PDE_WRITE_THROGH,
	PDE_NO_CACHE,
	PDE_ACCESSED,
	PDE_RESERVED,	/* Set to 0 */
	PDE_4MB,
	PDE_GLOBAL,	/* Ignored */
	PDE_RES1,
	PDE_RES2,
	PDE_RES3,
	PDE_RES4,
}pde_flags;

enum {
	PTE_PRESENT,
	PTE_RW,
	PTE_SUPERVISOR,
	PTE_WRITE_THROUGH,
	PTE_NO_CACHE,
	PTE_ACCESSED,
	PTE_DIRTY,
	PTE_PTAI,
	PTE_GLOBAL,
	PTE_LOCKED,	/* Permanently locked in. */
	PTE_DMA,	/* Suitable for ISA DMA */
	PTE_RES3,
	PTE_RES4,
}pte_flags;

#define GFP_KERNEL	0
#define GFP_DMA		(1<<PTE_DMA)

/* Functions */
void paging_init(void);
void paging_enable(void);
void pagefault(u_int32_t);

/* Lock/unlock one page */
void lock_page(u_int32_t);
void unlock_page(u_int32_t);

/* Sloppy function to lock some memory */
void lock_region(char *, u_int32_t, u_int32_t);

/* Exhaustively search memory for a free page */
u_int32_t get_free_pages(u_int32_t, int);

#endif /* __ARCH_IA32_MM__ */
