#include <kernel.h>
#include <arch/dma.h>

#define _DMA_PREFIX "      (dma): "

dma_t dma_chan[]={
	/* 8 bit DMA channels */
	{0x87,	0x00,	0x01,	0x0a, 	0x0b, 	0x0c},
	{0x83,	0x02,	0x03,	0x0a, 	0x0b, 	0x0c},
	{0x81,	0x04,	0x05,	0x0a, 	0x0b, 	0x0c},
	{0x82,	0x06,	0x07,	0x0a, 	0x0b, 	0x0c},

	/* 16 bit DMA channels */
	{0x8f,	0xc0,	0xc2,	0xd4, 	0xd6, 	0xd8},
	{0x8b,	0xc6,	0xc6,	0xd4, 	0xd6, 	0xd8},
	{0x89,	0xc8,	0xca,	0xd4, 	0xd6, 	0xd8},
	{0x8a,	0xcc,	0xce,	0xd4, 	0xd6, 	0xd8},
/* 	{page, 	addr, 	count, 	mask,	mode, 	clear}, */
};

void dma_cmd(u_int8_t chan, u_int8_t page, u_int16_t ofs, u_int16_t len, u_int8_t mode)
{
	/*
	kprint(_DMA_PREFIX "CMD: chan=%u page=%u ofs=0x%x len=%u mode=0x%x\n", 
		chan, page, ofs, len, mode);
	*/

	/* Its crucial we aren't interrupted */
	cli();

	/* Mask that sucker */
	outb(dma_chan[chan].mask, 0x04|chan);

	/* Clear any current transfers */
	outb(dma_chan[chan].clear, 0x00);

	/* Send the mode to the controller */
	outb(dma_chan[chan].mode, mode);

	/* Send the offset address byte at a time */
	outb(dma_chan[chan].addr, ofs&0x00FF);
	outb(dma_chan[chan].addr, (ofs&0xFF00)>>8);

	/* Send the page */
	outb(dma_chan[chan].page, page);
	
	/* Send data length */
	outb(dma_chan[chan].count, len&0x00FF);
	outb(dma_chan[chan].count, (len&0xFF00)>>8);
	
	/* unmask her */
	outb(dma_chan[chan].mask, chan);
	
	/* All done */
	sti();
	return;
}

void dma_write(u_int8_t chan, void *addr, u_int16_t len)
{
	u_int8_t page;
	u_int16_t ofs;
	u_int32_t a=(u_int32_t)addr;
	
	page = a >> 16;
	ofs  = a & 0xFFFF;

	dma_cmd(chan, page, ofs, len-1, 0x48+chan);
}

void dma_read(u_int8_t chan, void *addr, u_int16_t len)
{
	u_int8_t page;
	u_int16_t ofs;
	u_int32_t a=(u_int32_t)addr;
	
	page = a >> 16;
	ofs  = a & 0xFFFF;

	dma_cmd(chan, page, ofs, len-1, 0x44+chan);
}

void dma_init(void)
{
}

u_int32_t dma_probe(void)
{
	return 1;
}

u_int32_t dma_setup(void)
{
	return 1;
}
