#include <kernel.h>
#include <arch/smp.h>

cpu_feature_t __initdata features[]={
	{1,	"FPU"},
	{1<<7,	"Machine Exeption"},
	{1<<8,	"64bit compare and exchange"},
	{1<<9,	"APIC"},
	{1<<11,	"SEP"},
	{1<<12,	"MTRR"},
	{1<<13,	"PGE"},
	{1<<14,	"MCA"},
	{1<<15,	"CMOV"},
	{1<<16,	"PAT"},
	{1<<17,	"PSE"},
	{1<<18,	"PSN"},
	{1<<19,	"Cache Line Flush Instruction"},
	{0, NULL},
};

smp_info_t smpinfo={
	boot_cpu:0,
	present_cpus:0x00000000
};
mps_region_t __initdata regions[]={
	{0x00000, 1024},	/* mem_lo-1024 */
	{0xf0000, 0x10000},
	{0x9fc00, 1024},	/* EBDA address */
	{0,0}
};

/* The job for this function is simply to detect
* an SMP motherboard (even with just 1 CPU) so that
* we know early on whether to use PIC or APIC */
int __init probe_smp (void)
{
	int i;
	void *begin, *end;
	u_int32_t mp_sig=('_' | ('M'<<8) | ('P'<<16) | ('_'<<24)); /* "_MP_" */
	
	regions[0].base=boot_info.mem_lo-1024;

	for (i=0; regions[i].length; i++) {
		begin=(void *)regions[i].base;
		end=(void *)regions[i].base+regions[i].length;

		for(; begin<end; begin++) {
			if ( *((u_int32_t *)begin) == mp_sig)
				goto found;
		}
	}return 0;
	
found:
	smp_read_imps(begin);
	return 1;
}

u_int32_t __init smp_count_cpus(void *start, int16_t count)
{
	u_int32_t ptr=(u_int32_t)start;
	int16_t c=count;
	p_imps_cpu cpu;
	
	u_int32_t cpu_mask=0x00000000;
	u_int32_t cpu_count=0;
	
	while ( c-- ){
		switch ( *((unsigned char *)ptr) ) {
			case 0:
				cpu=(p_imps_cpu)ptr;

				if ( cpu->apic_id > MAX_CPU ) {
					kprint("Only 32 CPUs supported!\n");
				}
				
				smpinfo.cpu[cpu->apic_id].features=cpu->features;
				cpu_mask |= (1<<(cpu->apic_id));
				cpu_count++;
				
				if ( (cpu->flags & 2) )
					smpinfo.boot_cpu=cpu->apic_id;

				ptr+=12;
				break;
			default:
		}
		ptr+=8;
	}

	smpinfo.cpu_count=cpu_count;
	smpinfo.present_cpus=cpu_mask;
	
	return cpu_mask;
}

/* Take the info we want and save it. Assume that the 
* all SMP info will be overwritten after this function has
* exited! */
void __init smp_read_imps(p_imps_fps p)
{
	char *tmp;
	u_int32_t table;
	
	/* These strings are not null terminated, but padded
	* with spaces, grr */
	memcpy(&smpinfo.oem_id, p->cth->oem_id,8);
	memcpy(&smpinfo.prod_id, p->cth->prod_id,12);
	
	tmp=(char *)&smpinfo.oem_id;
	smpinfo.oem_id[8]='\0';
	while ( *tmp++ ) {
		if ( *tmp == ' ' ) {
			*tmp='\0';
			break;
		}
	}
	tmp=(char *)&smpinfo.prod_id;
	smpinfo.prod_id[12]='\0';
	while ( *tmp++ ) {
		if ( *tmp == ' ' ) {
			*tmp='\0';
			break;
		}
	}
	
	table=(u_int32_t)p->cth;
	table+=sizeof(imps_cth_t);
	smpinfo.local_apic=p->cth->lapic_addr;
	
	/* Detect all the CPUs */
	smp_count_cpus((void *)table, p->cth->entry_count);
	
	kprint("  (smp/mps): MPS found at 0x%x (BIOS version 1.%u) (%s/%s)\n",
		p, p->rev, &smpinfo.oem_id, &smpinfo.prod_id);
	kprint("  (smp/mps): %u CPUs detected. Boot CPU has ID %u\n",
		smpinfo.cpu_count, smpinfo.boot_cpu);
	kprint(" (smp/apic): Local APIC at address 0x%x\n",
		smpinfo.local_apic);
}
