#include <kernel.h>
#include <arch/timer.h>
#include <arch/idt.h>
#include <arch/8259a.h>
#include <lib/time.h>

volatile u_int32_t ticks=0;

void timer_isr(void)
{
	ticks++;
	irq_eoi(0);
}

void _irq0(void);
asm(
".globl _irq0		\n"
"_irq0:			\n"
"	pusha		\n"
"	call timer_isr	\n"
"	popa		\n"	
"	iret		\n"
);

/* Delay loop  */
void __delay(u_int32_t loops)
{
	int d0;
	__asm__ __volatile__(
		"       jmp 1f	\n"
		".align 16	\n"
		"1:     jmp 2f	\n"
		".align 16	\n"
		"2:     decl %0	\n"
		"       jns 2b	\n"
		:"=&a" (d0)
		:"0" (loops));
}

void __init pit_init(void)
{
	u_int32_t lb, lp=8;
	
	/* Coarse calibration */
	delayinfo.lpms=(1<<12);
	while ( delayinfo.lpms <<= 1 ) {
		set_timer_chan_oneshot(0xFFFF, 0);
		__delay(delayinfo.lpms);
		if ( get_timer_chan(0,1)<64000 ) break;
	}

	delayinfo.lpms>>=1;
	lb=delayinfo.lpms;

	/* Precision calculation */
	while (lp-- && (lb>>=1) ) {
		delayinfo.lpms |= lb;
		set_timer_chan_oneshot(0xFFFF, 0);
		__delay(delayinfo.lpms);
		if ( get_timer_chan(0,1) < 64000 ) {
			delayinfo.lpms &= ~lb;
		}

	}

	/* Normalise the results */
	delayinfo.delay=__delay;
	delayinfo.lpms *= PIT_SECOND / (65535-64000);
	delayinfo.lpms /= 1000;
}

void pit_start_timer1(void)
{
	//set_timer_chan(0xf8d6, 0);
	set_timer_chan(LATCH, 0);
	idt_interrupt(_irq0, VECTOR_M+0);
	irq_on(0);
}

void set_timer_chan(u_int16_t h, u_int8_t chan)
{
	outb_p(TMR_PORT, (chan*0x40) | PIT_BOTH | PIT_MODE_3);
	outb_p((0x40+chan), (u_int8_t)(h&0xFF));
	outb_p((0x40+chan), (u_int8_t)(h>>8));
}

void set_timer_chan_oneshot(u_int16_t h, u_int8_t chan)
{
	outb_p(TMR_PORT, (chan*0x40) | PIT_BOTH | PIT_MODE_0);
	outb_p((0x40+chan), (u_int8_t)(h&0xFF));
	outb_p((0x40+chan), (u_int8_t)(h>>8));
}

u_int32_t get_timer_chan(u_int8_t chan, int reset)
{
	u_int32_t x;
	
	outb_p(TMR_PORT, (chan*0x40) | (reset)? 0x0 : PIT_LATCH);
	x=inb_p(0x40+chan);
	x+=(inb_p(0x40+chan)<<8);
	return x;
}
