#include <kernel.h>
#include <arch/multiboot.h>
#include <arch/vga_bios.h>
#include <arch/setup.h>

kboot_t boot_info;

extern u_int32_t	_text;
extern u_int32_t	_end;
extern u_int32_t	__ksym_start;
extern u_int32_t	__ksym_end;

/* Initialises everything needed to begin the ia32 setup()
* function. */
void __init multiboot_init(unsigned long magic, unsigned long addr)
{
	multiboot_info_t	*mbi=(multiboot_info_t *)addr;
	
	/* Detect monitor type */
	switch (*(((u_int8_t *)0x410)) & 0x30) {
		case 0x00:
			boot_info.monitor=MONITOR_NONE;
			break;
		case 0x20:
			boot_info.monitor=MONITOR_COLOUR;
			break;
		case 0x30:
			boot_info.monitor=MONITOR_MONO;
			break;
		default:
			boot_info.monitor=MONITOR_UNKNOWN;
	}

	/* Initialise the VGA BIOS driver for
	* text mode output */
	vga_bios_init();

	/* Am I booted by a Multiboot-compliant boot loader?  */
	if (magic != MULTIBOOT_BOOTLOADER_MAGIC) {
		kprint("PANIC(mb): Invalid magic number: 0x%x\n", (unsigned) magic);
		return;
	}

	if ( MASK(mbi->flags, MBF_AOUT)  && MASK(mbi->flags, MBF_ELF) ) {
		kprint("PANIC(mb): Kernel can't be ELF AND AOUT!\n");
		return;
	}

	if ( !MASK(mbi->flags, MBF_MEM) ) {
		kprint("PANIC(mb): ScaraOS relies on bootloader counting memory\n");
		return;
	}

	if ( MASK(mbi->flags, MBF_CMDLINE) ) {
		boot_info.cmd_line=mbi->cmdline;
	}else{
		kprint("WARNING(mb): Bootloader didn't pass command line\n");
		boot_info.cmd_line="";
	}

	boot_info.mem_lo=mbi->mem_lower;
	boot_info.mem_up=mbi->mem_upper;
	boot_info.boot_method=BM_MULTIBOOT;

	boot_info.kernel_addr	=(u_int32_t)&_text;
	boot_info.kernel_end	=(u_int32_t)&_end;
	boot_info.ksym_start	=(u_int32_t)&__ksym_start;
	boot_info.ksym_end	=(u_int32_t)&__ksym_end;
	
	setup();
	reboot();
	return; /* Just incase exit() doesn't work */
}
