#include <fs/vfs.h>
#include <fs/ext2.h>
#include <arch/fdc.h>

#define _EXT2_PREFIX "EXT2: "

/* Ext2 disk structures */
ext2_super_t 		super;			/* Superblock */
ext2_group_desc_t 	bg[32];			/* Block group descriptors (1 blocks worth) */
u_int32_t 		bitmap_block[256];	/* Data block bitmaps */
u_int32_t 		bitmap_inode[256];	/* Inode block bitmaps */
ext2_inode_t 		inodes[8];		/* Inode table (1 blocks worth) */
char 			block[1024];		/* Data block buffer */
p_ext2_dirent		dir;

int ext2_check_super(p_ext2_super super)
{
	/* Do some quick checks to stop us destroying the filesytstem */
	if ( super->s_log_block_size ) {
		kprint(_EXT2_PREFIX "PANIC: Only 1KB blocksize supported currently\n");
		return 0;
	}
	if ( super->s_magic != EXT2_SUPER_MAGIC ) {
		kprint(_EXT2_PREFIX "PANIC: Bad magic number in superblock!\n");
		return 0;
	}
	return 1;
}

void ext2_read_super(p_inode i)
{
	file_t f;
	int j;

	f.f_op=&blk_ops;
	
	/* Open the device */
	if ( f.f_op->open(i, &f) <0 ) {
		kprint(_EXT2_PREFIX "open() failed for device (fd0)\n");
		return;
	}
	
	/* Seek past the boot block */
	f.f_op->lseek(&f, 1024, 0);
	
	/* Read in the superblock */
	if ( f.f_op->read(&f, (char *)&super, sizeof(ext2_super_t)) <0 ) {
		kprint(_EXT2_PREFIX "read(super) failed for device (fd0)");
		f.f_op->close(&f);
		return;
	}

	/* Check the superblock */
	if ( !ext2_check_super(&super) ) {
		f.f_op->close(&f);
		return;
	}

	/* Read in the first block group */
	if ( f.f_op->read(&f, (char *)&bg, sizeof(bg)) <0 ){
		kprint(_EXT2_PREFIX "read(bg) failed for device (fd0)");
		f.f_op->close(&f);
		return;
	}

	kprint(_EXT2_PREFIX "Group 0: block bitmap at %u, inode bitmap at %u, inode table at %u\n",
		bg[0].bg_block_bitmap,
		bg[0].bg_inode_bitmap,
		bg[0].bg_inode_table);
	
	/* Read the block bitmaps */
	f.f_op->lseek(&f, bg[0].bg_block_bitmap*1024, SEEK_SET);
	f.f_op->read(&f, (char *)&bitmap_block, 1024);
	
	/* Read the inode bitmaps */
	f.f_op->lseek(&f, bg[0].bg_inode_bitmap*1024, SEEK_SET);
	f.f_op->read(&f, (char *)&bitmap_inode, 1024);

	/* Read the inode table */
	f.f_op->lseek(&f, bg[0].bg_inode_table*1024, SEEK_SET);
	f.f_op->read(&f, (char *)&inodes, 1024);

	kprint(_EXT2_PREFIX "Root Dir (/): %u bytes, first data block is %u\n",
		inodes[1].i_size, inodes[1].i_block[0]);
	
	/* Read in the first data block */
	f.f_op->lseek(&f, inodes[1].i_block[0]*1024, SEEK_SET);
	f.f_op->read(&f, (char *)&block, 1024);
	
	kprint(_EXT2_PREFIX "Root Dir (/): ");
	
	/* List all the files */
	for(j=0; j<1024; ) {
		dir=(void *)&block;
		(u_int32_t)dir += j;
		kprint("%s ", (char *)&dir->name);
		j+=dir->rec_len;
	}
	
	/* Close the device */
	f.f_op->close(&f);
}
