#include <kernel.h>
#include <fs/vfs.h>
#include <arch/fdc.h>

/* 
* vfs_block.c: 
* A Generic block device driver. Requires nothing from the low level
* driver other than one block_rw function. This is for simple block
* devices like disks and suchlike.
*/

/* Devices we handle */
blkdev_t bdev[]={
	{1,	0,	fdc_blk_rw,	9},
};

p_blkdev blk_get_dev(u_int32_t maj, u_int32_t min)
{
	u_int32_t i;

	for (i=0; i<sizeof(bdev)/sizeof(blkdev_t); i++) {
		if ( bdev[i].major == maj ) {
			return &bdev[i];
		}
	}

	return NULL;
}

int32_t blk_open(p_inode i, p_file f)
{
	f->f_pos=0;
	f->priv=(void *)blk_get_dev(i->major, i->minor);
	return (f->priv) ? -ESUCCESS : -ENODEV;
}

int32_t blk_close(p_file f)
{
	return -ESUCCESS;
}

int32_t blk_read(p_file f, char *buf, ofs_t len)
{
	p_blkdev b=(p_blkdev)f->priv;
	u_int32_t start, count;

	if ( f->f_pos % (1<<b->blk_size) ) {
		kprint("Offset is not multiple of block size\n");
		return -EPARAM;
	}
	
	if ( len % (1<<b->blk_size) ) {
		kprint("Length is not multiple of block size\n");
		return -EPARAM;
	}
	
	
	start=f->f_pos/(1<<b->blk_size);
	count=(len/(1<<b->blk_size));
	fdc_blk_rw(RWB_READ, start, buf, count);
	f->f_pos+=len;

	return len;
}

int32_t blk_write(p_file f, char *buf, ofs_t len)
{
	return -ESUCCESS;
}

int32_t blk_lseek(p_file f, ofs_t pos, u_int32_t whence)
{
	switch (whence) {
		case SEEK_CUR:
			f->f_pos += pos;
			break;
		case SEEK_SET:
			f->f_pos = pos;
			break;
		default:
			return -EPARAM;
	}
	
	return -ESUCCESS;
}

file_ops_t blk_ops={
	open:	blk_open,
	close:	blk_close,
	read:	blk_read,
	write:	blk_write,
	lseek:	blk_lseek
};

