#ifndef __EXT2_HEADER_INCLUDED__
#define __EXT2_HEADER_INCLUDED__

#include <types.h>

#define EXT2_SUPER_MAGIC	0xef53
#define EXT2_MIN_BLOCK_SIZE	1024
#define EXT2_MAX_BLOCK_SIZE	4096
#define EXT2_N_BLOCKS		15
#define EXT2_NAME_LEN		255

/* A super block */
typedef struct ext2_suber_block{
	/* Standard flags */
	u_int32_t	s_inodes_count;
	u_int32_t	s_blocks_count;
	u_int32_t	s_r_blocks_count;
	u_int32_t	s_free_blocks_count;
	u_int32_t	s_free_inodes_count;
	u_int32_t	s_first_data_block;
	u_int32_t	s_log_block_size;
	u_int32_t	s_log_frag_size;
	u_int32_t	s_blocks_per_group;
	u_int32_t	s_frags_per_group;
	u_int32_t	s_inodes_per_group;
	u_int32_t	s_mtime;
	u_int32_t	s_wtime;
	u_int16_t	s_mnt_count;
	u_int16_t	s_max_mnt_count;
	u_int16_t	s_magic;
	u_int16_t	s_state;
	u_int16_t	s_errors;
	u_int16_t	s_minor_rev_level;
	u_int32_t	s_lastcheck;
	u_int32_t	s_checkinterval;
	u_int32_t	s_creator_os;
	u_int32_t	s_rev_level;
	u_int16_t	s_def_resuid;
	u_int16_t	s_def_resgid;

	/* EXT2_DYNAMIC_REV */
	u_int32_t	s_first_ino;
	u_int16_t	s_inode_size;
	u_int16_t	s_block_group_nr;
	u_int32_t	s_feature_compat;
	u_int32_t	s_feature_incompat;
	u_int32_t	s_ro_compat;
	u_int8_t	s_uuid[16];
	int8_t		s_volume_name[16];
	int8_t		s_last_mounted[64];
	u_int32_t	s_algorithm_usage_bitmap;

	/* Performance hints */
	u_int8_t	s_prealloc_blocks;
	u_int8_t	s_prealloc_dir_blocks;
	u_int16_t	s_padding1;
	u_int32_t	s_reserved[204];
}ext2_super_t, *p_ext2_super;

/* A block group descriptor */
typedef struct ext2_group_desc{
	u_int32_t	bg_block_bitmap;
	u_int32_t	bg_inode_bitmap;
	u_int32_t	bg_inode_table;
	u_int16_t	bg_free_blocks_count;
	u_int16_t	bg_free_inodes_count;
	u_int16_t	bg_used_dirs_count;
	u_int16_t	bg_pad;
	u_int32_t	bg_reserved[3];
}ext2_group_desc_t, *p_ext2_group_desc;

typedef struct ext2_inode{
	u_int16_t	i_mode;		/* File mode */
	u_int16_t	i_uid;		/* Low 16 bits of Owner Uid */
	u_int32_t	i_size;		/* Size in bytes */
	u_int32_t	i_atime;	/* Access time */
	u_int32_t	i_ctime;	/* Creation time */
	u_int32_t	i_mtime;	/* Modification time */
	u_int32_t	i_dtime;	/* Deletion Time */
	u_int16_t	i_gid;		/* Low 16 bits of Group Id */
	u_int16_t	i_links_count;	/* Links count */
	u_int32_t	i_blocks;	/* Blocks count */
	u_int32_t	i_flags;	/* File flags */
	u_int32_t	i_reserved;
	u_int32_t	i_block[EXT2_N_BLOCKS];
	u_int32_t	i_generation;
	u_int32_t	i_file_acl;
	u_int32_t	i_dir_acl;
	u_int32_t	i_faddr;
	union {
		struct {
			u_int8_t	l_i_frag;
			u_int8_t	l_i_fsize;
			u_int16_t	l_i_pad1;
			u_int16_t	l_i_uid_high;
			u_int16_t	l_i_gid_high;
			u_int32_t	l_i_reserved2;
		}linux2;
	}osd2;
}ext2_inode_t, *p_ext2_inode;

typedef struct ext2_dir_entry_2 {
	u_int32_t	inode;			/* Inode number */
	u_int16_t	rec_len;		/* Directory entry length */
	u_int8_t	name_len;		/* Name length */
	u_int8_t	file_type;
	char		name[EXT2_NAME_LEN];	/* File name */
}ext2_dirent_t, *p_ext2_dirent;

#endif /* __EXT2_HEADER_INCLUDED__ */
